%   Description: Takes a polynomial, and computes the tropical roots.
%   Inputs: 
    %   p=[p_d,...,p_1,p_0]: a polynomial
    %   demo: when demo is 1, a convexhull which is used to compute the tropical roots will be demosntrated.    
%   Output: 
    %   y: an n by 2 matrix, the second column presents the list of
    %   tropical roots while the first column presents the multiplicity of
    %   the corresponding tropical roots.    
function y=tropical_roots(p,demo)       
    % check the input format
    if size(p,1)>1
        error('The input polynomial should be of the format [a_n,a_{n-1},..]');
        return;
    end
    if nnz(p) ==0
        error('The input polynomial is null!'); 
        return;
    end
    while p(1)==0
        p(1)=[];
    end
    n=size(p,2);
    if n==1
        error('The input polynomial is constant');
        return;
    end  
    zeroroots=0;
    while p(n-zeroroots)==0        
        p(n-zeroroots)=[];
        zeroroots=zeroroots+1;
    end   
    if nnz(p)==1
        y=[zeroroots,0];
        return
    end
    % 
    n=size(p,2);
    eps=10^(-14);
    j=1;  
    for i= n:-1:1
        if p(i) ~= 0 
           v_lst(j,:)=[n-i,log(abs(p(i)))];           
           j=j+1; 
        end
    end   
    if demo==1
        clf;
        plot(v_lst(:,1),v_lst(:,2),'ro','MarkerFaceColor','r','MarkerSize',5);
        hold on;
    end 
    
    n=size(v_lst,1);
    m=(v_lst(n,2)-v_lst(1,2))/(v_lst(n,1)-v_lst(1,1));    
    t1(1,1)=v_lst(1,2);
    t1(2:n-1,1)=v_lst(1,2)+m*(v_lst(2:n-1,1)-v_lst(1,1));
    t1(n,1)=v_lst(n,2);
    t2=(t1>v_lst(:,2))& (abs(t1-v_lst(:,2))>eps);
    v_lst(t2,:)=[];    
    v_lst=graham_scan(v_lst);  
    if demo==1
         plot(v_lst(:,1),v_lst(:,2),'b','LineWidth',3);    
         hold off;
    end
    for i= 1:(size(v_lst,1)-1) 
        tr=exp(-(( v_lst(i,2)- v_lst((i+1),2))/( v_lst(i,1)- v_lst(i+1,1))));  
        if (abs(tr-ceil(tr))<eps) & (ceil(tr)~=0)
            tr=ceil(tr);
        end
        if (abs(tr-floor(tr))<eps) & (floor(tr)~=0)
            tr=floor(tr);
        end           
        y(i,:)=[v_lst(i+1,1)-v_lst(i,1),tr];
    end  
    if zeroroots~=0
        y=[zeroroots,0;y];
    end
end
